@@ -181,7 +181,9 @@ static int select_table(mpeg3_layer_t *audio)
 	return 0;
 }
 
-static int step_one(mpeg3_layer_t *audio, unsigned int *bit_alloc, int *scale)
+static int step_one(mpeg3_layer_t *audio, 
+	unsigned int *bit_alloc, 
+	int *scale)
 {
    	int stereo = audio->channels - 1;
     int sblimit = audio->II_sblimit;
@@ -191,20 +193,26 @@ static int step_one(mpeg3_layer_t *audio, unsigned int *bit_alloc, int *scale)
     int i, result = 0;
     unsigned int *scfsi_buf = audio->layer2_scfsi_buf;
     unsigned int *scfsi, *bita;
+	unsigned int *bita_end = bit_alloc + 64;
     int sc, step;
 
     bita = bit_alloc;
     if(stereo)
     {
 /* Stereo */
-    	for(i = jsbound; i; i--, alloc1 += (1 << step))
+    	for(i = jsbound; 
+			i && bita < bita_end - 2; 
+			i--, alloc1 += (1 << step))
     	{
         	*bita++ = (char)mpeg3bits_getbits(audio->stream, step = alloc1->bits);
         	*bita++ = (char)mpeg3bits_getbits(audio->stream, step);
     	}
-    	for(i = sblimit - jsbound; i; i--, alloc1 += (1 << step))
+    	for(i = sblimit - jsbound; 
+			i && bita < bita_end - 2; 
+			i--, alloc1 += (1 << step))
     	{
-        	bita[0] = (char)mpeg3bits_getbits(audio->stream, step = alloc1->bits);
+        	bita[0] = (char)mpeg3bits_getbits(audio->stream, 
+				step = alloc1->bits);
         	bita[1] = bita[0];
         	bita += 2;
     	}
@@ -377,7 +385,7 @@ int mpeg3audio_dolayer2(mpeg3_layer_t *audio,
 	int scale[192];
 	int single = audio->single;
 	int output_position = 0;
-
+//printf("mpeg3audio_dolayer2 1\n");
 
 	frame += 4;
 /* Set up bitstream to use buffer */
